<?php

namespace Drupal\fluent;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldType\EntityReferenceItem;
use Drupal\field\Entity\FieldConfig;
use Drupal\fluent\Plugin\FluentFieldResolverManager;

class FieldResolver {

  private array $fieldResolvers = [];

  private $entity;

  /**
   * @var \Drupal\fluent\FieldResolution
   */
  private FieldResolution $resolution;

  public function __construct(FluentFieldResolverManager $plugin) {
    $this->pluginManager = $plugin;

    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      $this->fieldResolvers[$id] = $this->pluginManager->createInstance($id);
    }

    $this->commonResolver = $this->fieldResolvers['common_field'];
    unset($this->fieldResolvers['common_field']);
  }

  public function resolve($entity, array $path, int $position) {
    $this->entity = $entity;
    $this->resolution = new FieldResolution();

    // Save the current entity.
    $tmpEntity = $this->entity;
    $items = count($path);
    $count = 0;

    do {
      $possibleField = $path[$count];

      $solvedField = $this->getResolver($tmpEntity, $possibleField, $position);

      if ($solvedField === NULL) {
        break;
      }

      $tmpEntity = $solvedField;

      if (is_array($solvedField)) {
        break;
      }

      if ($this->skipCount($solvedField)) {
        continue;
      }

      $count++;
    } while ($count < $items);

    return $this->resolution;
  }

  private function skipCount($solvedField): bool {
    return (
      $solvedField instanceof FieldItemListInterface
      ||
      $solvedField instanceof EntityReferenceItem
    );
  }

  private function getResolver($entity, $name, $position = 0) {
    if (method_exists($entity, 'getFieldDefinition')) {
      /** @var FieldConfig $fieldDefinition */
      $fieldDefinition = $entity->getFieldDefinition($name);

      if ($fieldDefinition) {
        $this->resolution->setFieldDefinition($fieldDefinition);

        foreach ($this->fieldResolvers as $resolver) {
          if ($resolver->can($fieldDefinition)) {
            $cardinality = $fieldDefinition->getFieldStorageDefinition()->getCardinality();

            $field = $resolver->handler($entity, $name, [
              'position' => $position,
              'field_cardinality' => $cardinality,
            ]);
            $this->resolution->setField($field);

            return $field;
          }
        }
      }

      $field = $this->commonResolver->handler($entity, $name, ['position' => $position]);
      $this->resolution->setField($field);

      return $field;
    } else {
      // TODO: don't fieldable entity
    }
  }
}
