<?php

namespace Drupal\fluent;

use Drupal\Core\Entity\EntityBase;

/**
 *
 */
class Fluent {

  /**
   * @var mixed
   */
  private $entity;

  /**
   * @var \Drupal\fluent\FieldItemResolver
   */
  private FieldItemResolver $fieldItemResolver;

  /**
   * @var \Drupal\fluent\FieldResolver
   */
  private FieldResolver $fieldResolver;

  /**
   *
   */
  public function __construct(FieldResolver $fieldResolver, FieldItemResolver $fieldItemResolver) {
    $this->fieldResolver = $fieldResolver;
    $this->fieldItemResolver = $fieldItemResolver;
  }

  /**
   * @param mixed $entity
   *  Fieldable entity.
   *
   * @return $this
   */
  public function using($entity) {
    $this->entity = $entity;

    return $this;
  }

  /**
   * @param string $path
   * @param mixed $defaultValue
   *
   * @return mixed
   */
  public function value(string $path, $defaultValue = NULL) {
    $resolution = $this->fieldResolution($path, 0, $defaultValue);

    if ($resolution->isMultiValue()) {
      throw new \Exception('Field is multi-value, use values, nthValue or firstValue methods');
    }

    return $resolution->getValue();
  }

  /**
   * @param string $path
   * @param $defaultValue
   *
   * @return \Drupal\fluent\Collection
   */
  public function firstValue(string $path, $defaultValue = NULL) {
    return $this->nthValue($path, 0, $defaultValue)->first();
  }

  /**
   * @param string $path
   * @param array $default
   *
   * @return \Drupal\fluent\Collection
   */
  public function values(string $path, array $default = []): Collection {
    return $this->nthValue($path, -1, $default);
  }

  /**
   * @param string $path
   * @param int $position
   * @param $defaultValue
   *
   * @return \Drupal\fluent\Collection
   */
  public function nthValue(string $path, $position, $defaultValue = NULL): Collection {
    $resolution = $this->fieldResolution($path, $position, $defaultValue);

    if (!$resolution->isMultiValue()) {
      throw new \Exception('Field is not multi-value, use value() method');
    }

    $field = $resolution->getValue();

    if ($field instanceof EntityBase) {
      $collection = [$field];
    } else {
      $collection = $field;
    }

    return Collection::make($collection);
  }

  private function fieldResolution($path, $position, $defaultValue) {
    $resolution = $this->fieldResolver->resolve($this->entity, $this->getPath($path), $position);
    return $this->fieldItemResolver->resolve($resolution, $this->getPath($path), $defaultValue);
  }

  private function getPath(string $path): array {
    return explode('.', $path);
  }
}
